# coding: utf-8
import pandas as pd


def stochastic_k(prices, k):
    l = pd.rolling_min(prices, k)
    h = pd.rolling_max(prices, k)
    return 100 * (prices - l) / (h - l)


def stochastic_d(prices, k, d):
    return pd.rolling_mean(stochastic_k(prices, k), d)


def stochastic_sd(prices, k, d, sd):
    # (k, d, sd) = (14, 3, 3)
    return pd.rolling_mean(stochastic_d(prices, k, d), sd)


def bollinger_band(prices, period, sigma):
    mean = pd.rolling_mean(prices, period)
    std = pd.rolling_std(prices, period)
    return mean + sigma * std


def macd_line(series, fast=26, slow=12, signal=9):
    # Chris Manning (fast, slow, signal) = (17, 9, 7)
    return pd.ewma(series, span=slow) - pd.ewma(series, span=fast)


def macd_signal(series, fast=26, slow=12, signal=9):
    return pd.ewma(macd_line(series, fast=fast, slow=slow, signal=signal), span=signal)


def rsi(prices, period=14):
    # fill 0 at the first day
    diff = (prices - prices.shift(1)).fillna(0)

    def calc(p):
        gain =  p[p > 0].sum() / period
        loss = -p[p < 0].sum() / period
        rs = gain / loss
        return 100 - 100/(1+rs)

    return pd.rolling_apply(diff, period, calc)



# coding: utf-8
import calendar
import datetime

from dateutil import relativedelta


def last_date():
    """株の最後の日を返す"""
    # for JST
    now = datetime.datetime.today() + relativedelta.relativedelta(hours=9)
    weekday = now.weekday()
    if weekday in [calendar.SUNDAY, calendar.SATURDAY]:
        dt = now + relativedelta.relativedelta(weekday=relativedelta.FR(-1))
    else:
        dt = now - relativedelta.relativedelta(days=1)
    return dt.date()



# coding: utf-8
import pandas as pd

from stock import query

from logging import getLogger
logger = getLogger(__name__)


def make_data_frame(day_info_query):
    return pd.DataFrame(
        [(info.w.js_datetime, info.w.closing, info.w.volume,
            info.w.opening, info.w.high, info.w.low)
            for info in day_info_query],
        columns=["date", "closing", "volume", "opening", "high", "low"]
    )


def get(company_id):
    q = query.DayInfo.get(company_id)
    from stock import wrapper
    return wrapper.DayInfoQuery(q)



# coding: utf-8
import sqlalchemy as sql
import pandas as pd

from stock import models
from stock import util
from stock import config as C

from logging import getLogger
logger = getLogger(__name__)


class Reader(object):

    header = C.EXCEL_COMPANY_HEADER

    def __init__(self, filepath):
        excel = pd.ExcelFile(filepath)
        if C.SHEET_NAME not in excel.sheet_names:
            raise ValueError("The sheet %s doesn't exist in %s" % (C.SHEET_NAME, filepath))
        sheet = excel.parse(C.SHEET_NAME)

        if not self._check_header(sheet.columns):
            raise ValueError("invalid header")

        # convert japanese keys to model keys
        self.sheet = sheet.rename(columns=util.dict_inverse(self.header))
        self.filepath = filepath
        self.session = models.Session()

    def _check_header(self, row):
        return list(row) == list(self.header.values())

    def iter(self):
        for d in self.sheet.T.to_dict().values():
            yield {k: v for k, v in d.items() if k in models.Company.__dict__}

    def store(self):
        # TODO: exlucde duplicated code
        for data in self.iter():
            try:
                ins = models.Company(**data)
            except ValueError as e:
                logger.warn("The company whose code is %s is not imported: %s" % (data.get("code"), e))
            else:
                self.session.add(ins)
        try:
            self.session.commit()
        except sql.exc.IntegrityError as e:
            logger.warn("Can't import company data")
            logger.warn("%s" % e)
            return False
        return True



# coding: utf-8
import sqlalchemy as sql
from sqlalchemy.ext.declarative import declarative_base

from . import config as C
from . import wrapper


engine = sql.create_engine(C.DATABASE_URL, **C.CREATE_ENGINE)
Base = declarative_base(bind=engine)
Session = sql.orm.sessionmaker(bind=engine)


class StockExchange(Base):

    __tablename__ = "stock_exchange"

    id = sql.Column(sql.Integer, primary_key=True)
    name = sql.Column(sql.Text, nullable=False)
    company = sql.orm.relationship(
        "Company",
        backref="stock_exchange",
        uselist=False
    )


class Price(Base):

    __tablename__ = "price"
    __table_args__ = (sql.UniqueConstraint('date', 'qcode'), )

    id = sql.Column(sql.Integer, primary_key=True)
    high = sql.Column(sql.Float, nullable=False)
    low = sql.Column(sql.Float, nullable=False)
    open = sql.Column(sql.Float, nullable=False)
    close = sql.Column(sql.Float, nullable=False)
    date = sql.Column(sql.Date, nullable=False)
    qcode = sql.Column(sql.String(64), nullable=False, index=True)
    quandl_code = sql.Column(sql.String(64), nullable=False, index=True)
    volume = sql.Column(sql.Integer, nullable=True)


class DayInfo(Base):

    __tablename__ = "day_info"
    __table_args__ = (sql.UniqueConstraint('date', 'company_id'), )

    id = sql.Column(sql.Integer, primary_key=True)
    high = sql.Column(sql.Float, nullable=False)
    low = sql.Column(sql.Float, nullable=False)
    opening = sql.Column(sql.Float, nullable=False)
    closing = sql.Column(sql.Float, nullable=False)
    date = sql.Column(sql.Date, nullable=False)
    volume = sql.Column(sql.Integer, nullable=False)
    company_id = sql.Column(
        sql.Integer,
        sql.ForeignKey("company.id", onupdate="CASCADE", ondelete="CASCADE"),
        nullable=False
    )
    company = sql.orm.relation(
        'Company',
        order_by='Company.id',
        uselist=False,
        backref='day_info_list'
    )

    def __str__(self):
        return "({company_id}, {opening}, {high}, {low}, {closing})".format(**self.to_dict())

    @property
    def w(self):
        return wrapper.DayInfo(self)


class SplitStockDate(Base):

    __tablename__ = "split_stock_date"
    __table_args__ = (sql.UniqueConstraint("date", "company_id"), )

    id = sql.Column(sql.Integer, primary_key=True)
    from_number = sql.Column(sql.Integer, nullable=False)
    to_number = sql.Column(sql.Integer, nullable=False)
    date = sql.Column(sql.Date, nullable=False)
    company_id = sql.Column(
        sql.Integer,
        sql.ForeignKey("company.id", onupdate="CASCADE", ondelete="CASCADE"),
        nullable=False
    )
    company = sql.orm.relation(
        "Company",
        order_by="Company.id",
        uselist=False,
        backref="split_stock_dates"
    )


class Company(Base):

    __tablename__ = "company"
    __table_args__ = (sql.UniqueConstraint("code"), )

    id = sql.Column(sql.Integer, primary_key=True)
    name = sql.Column(sql.Text, nullable=False)
    description = sql.Column(sql.Text, nullable=True)
    code = sql.Column(sql.Integer)
    category17 = sql.Column(sql.Integer)
    category33 = sql.Column(sql.Integer)
    scale = sql.Column(sql.Integer, nullable=False)
    stock_exchange_id = sql.Column(
        sql.Integer,
        sql.ForeignKey('stock_exchange.id', onupdate="CASCADE", ondelete="CASCADE"),
        nullable=True,
    )
    search_field = sql.orm.relation(
        "CompanySearchField",
        uselist=False,
        back_populates="company"
    )

    def __str__(self):
        return "({id}, {name}, {code})".format(**self.__dict__)

    @property
    def w(self):
        return wrapper.Company(self)

    @sql.orm.validates('scale')
    def validate_scale(self, key, scale):
        if isinstance(scale, int):
            return scale
        elif isinstance(scale, float):
            return int(scale)
        else:
            raise ValueError("%s is not allowed for scale." % scale)


class CurrentValue(Base):

    __tablename__ = "current_value"

    id = sql.Column(sql.Integer, primary_key=True)
    value = sql.Column(sql.Integer)
    datetime = sql.Column(sql.DateTime, nullable=False)


class CompanySearchField(Base):

    __tablename__ = "company_search_field"

    id = sql.Column(sql.Integer, primary_key=True)

    ratio_closing1_minus_closing2 = sql.Column(sql.Float, nullable=True, index=True)
    ratio_closing_minus_rolling_mean_25 = sql.Column(sql.Float, nullable=True, index=True)
    closing_rsi_14 = sql.Column(sql.Float, nullable=True, index=True)
    ratio_1sigma_low_minus_closing = sql.Column(sql.Float, nullable=True, index=True)
    ratio_2sigma_low_minus_closing = sql.Column(sql.Float, nullable=True, index=True)

    # MACD
    closing_macd_minus_signal1_26_12_9 = sql.Column(sql.Float, nullable=True, index=True)
    closing_macd_minus_signal2_26_12_9 = sql.Column(sql.Float, nullable=True, index=True)

    # stochastic
    closing_stochastic_d_minus_sd1_14_3_3 = sql.Column(sql.Float, nullable=True, index=True)
    closing_stochastic_d_minus_sd2_14_3_3 = sql.Column(sql.Float, nullable=True, index=True)

    # (0, 1 sigma) => 1, (1 sigma, 2 sigma) => 2 and so on
    interval_closing_bollinger_band_20 = sql.Column(sql.Integer, nullable=True, index=True)

    low_min_25 = sql.Column(sql.Float, nullable=True, index=True)
    low_min_75 = sql.Column(sql.Float, nullable=True, index=True)
    low_min_200 = sql.Column(sql.Float, nullable=True, index=True)

    company_id = sql .Column(
        sql.Integer,
        sql.ForeignKey('company.id', onupdate="CASCADE", ondelete="CASCADE"),
        nullable=True,
    )
    company = sql.orm.relation("Company", back_populates="search_field")

    @property
    def w(self):
        return wrapper.CompanySearchField(self)



# coding: utf-8
from flask import Flask, request, jsonify
from flask import render_template as _render_template

from stock import service
from stock import config as C


app = Flask(__name__)


def render_template(name, **kw):
    kw["service"] = service
    kw["C"] = C
    return _render_template(name, **kw)


def to_int(key):
    try:
        return int(request.args.get(key))
    except:
        return None


@app.route("/", methods=["GET"])
def index():
    field_keys = [
        "ratio_closing1_minus_closing2",
        "ratio_closing_minus_rolling_mean_25",
        "closing_rsi_14",
        "closing_macd_minus_signal",
        "interval_closing_bollinger_band_20",
        "closing_stochastic_d_minus_sd",
    ]
    d = {k: to_int(k) for k in field_keys}
    company_list = service.company.get(**d)
    return render_template('index.html', **{"company_list": company_list})


@app.route("/company/<int:id>", methods=["GET"])
def company(id):
    company = service.company.first(id=id, raise_404=True)
    return render_template('company.html', **{"company": company})


@app.route("/api/<int:company_id>", methods=["GET"])
def api(company_id):
    company = service.company.first(id=company_id, raise_404=True)
    q = service.day_info.get(company_id)
    return jsonify({
        "company": company.w.to_dict(),
        "series": q.to_series(),
    })


@app.route("/about", methods=["GET"])
def about():
    return render_template('about.html')



# coding: utf-8
import re

from .interface import Scraper, Interface, ParseError

REG_SPLIT_STOCK_DATE = re.compile(r"分割\W+(?P<from_number>\d+)株.*?(?P<to_number>\d+)株")
REG_DATE = re.compile(r"(?P<year>\d{4})年(?P<month>\d{1,2})月(?P<day>\d{1,2})日")


def _match_date(td):
    match = REG_DATE.match(td)
    if not match:
        raise ParseError("Not match date. %s" % td)
    return "{year}-{month}-{day}".format(**match.groupdict())


def parse_day_info(tr):
    tds = [t.text for t in tr.findAll("td")]
    if len(tds) == 2:
        return  # skip split stock date
    elif len(tds) != 7:
        raise ParseError("Can't parse day info. Need 7 columns. %s" % tr)
    return {
        "date": _match_date(tds[0]),
        "opening": tds[1],
        "high": tds[2],
        "low": tds[3],
        "closing": tds[4],
        "volume": tds[5],
    }


def parse_split_stock_date(tr):
    tds = [t.text for t in tr.findAll("td")]
    if len(tds) == 7:
        return  # skip day info
    elif len(tds) != 2:
        raise ParseError("Can't parse day info. Need 2 columns. %s" % tr)
    match = REG_SPLIT_STOCK_DATE.match(tds[1])
    if not match:
        raise ParseError("Not match split stock date. %s" % tds[1])
    return {
        "date": _match_date(tds[0]),
        "from_number": match.group("from_number"),
        "to_number": match.group("to_number"),
    }


class CurrentValue(Scraper):
    url = "http://stocks.finance.yahoo.co.jp/stocks/detail/?code={code}.T"

    def parse(self, soup):
        return soup.findAll("td", {"class": ["stoksPrice"]})[1].text


class History(Scraper):
    url = ("http://info.finance.yahoo.co.jp/history/?code={code}.T&"
           "sy={sy}&sm={sm}&sd={sd}&ey={ey}&em={em}&ed={ed}&tm=d&p={p}")

    def _parse(self, tr):
        return parse_day_info(tr)

    def parse(self, soup):
        span = soup.find(True, {"class": "stocksHistoryPageing"})
        if span is None:
            raise ParseError("Can't find css class stocksHistoryPageing")
        return [self._parse(tr) for tr
                in span.next_sibling.findAll("tr")[1:]]  # skip header


class SplitStockDate(History):

    def _parse(self, tr):
        return parse_split_stock_date(tr)


class YahooJapan(Interface):
    scrapers = [SplitStockDate, History, CurrentValue]



import datetime

import enum
import pandas as pd
import quandl

import matplotlib.pyplot as plt

from stock import models
from stock.service.simulate import RollingMean, MACD
from stock.service.simulate import timing


DEFAULT_ROLLING_MEAN_RATIO = 5

class PriceType(enum.Enum):
    open = "open"
    low = "low"
    high = "high"
    close = "close"


MAP_TO_COLUMNS = {
    "Open Price": "open",
    "Close Price": "close",
    "High Price": "high",
    "Low Price": "low",
}

# どれを一番良い結果として取るかも、問題となる
# 3か月売買での平均とかの方がいいかも(長期の計算は、次も当てはまるとは限らない)
# 最終結果だけでなく、平均的に儲けているものが、良いかも

# start/end dateを加える(index)

DATE_FORMATS = ["%Y/%m/%d", "%Y-%m-%d"]
def str_to_date(s):
    for fmt in DATE_FORMATS:
        try:
            return datetime.datetime.strptime(s, fmt).date()
        except:
            pass
    else:
        raise ValueError


def s(quandl_code="NIKKEI/INDEX", price_type=PriceType.close, way=None, lostcut=3, start=None, end=None, **kw):
    if isinstance(start, str):
        start = str_to_date(start)
    if isinstance(end, str):
        end = str_to_date(end)
    if not isinstance(price_type, enum.Enum):
        price_type = PriceType(price_type)
    df = get_from_quandl(quandl_code, last_date=None)
    series = getattr(df, price_type.name)
    series = series.ix[start:end]

    r = 0
    df = None
    # 一番儲けらるもの(パラメータの調節が必要なものもある.) and / or もできるようにしたい
    lists = [MACD(series)] + [RollingMean(series, i) for i in range(1, 10)]
    for l in lists:
        df_result = l.simulate_action()
        if df_result.empty:
            continue
        accumulation = df_result.ix[-1].accumulation
        if r < accumulation:
            r = max(r, accumulation)
            df = l
    return (r, df)


def create():
    models.Base.metadata.create_all()


def drop():
    models.Base.metadata.drop_all()


# 元のグラフも表示した方がわかりやすい
# or use `df.plot()`
def drow(df):
    plt.figure(figsize=(15, 15))
    pass



# 使用できるqcodeを表示できるの？

# interface
# PriceFrame(qcode="XXX").open.rolling_mean(25).plot()



def chart(qcode="NIKKEI/INDEX", price_type=PriceType.close, way=None, lostcut=3, **kw):
    if not isinstance(price_type, enum.Enum):
        price_type = PriceType(price_type)
    df = get_from_quandl(qcode, last_date=None)
    series = getattr(df, price_type.name)
    return MACD(series)
    return RollingMean(series, ratio=kw.get("ratio", DEFAULT_ROLLING_MEAN_RATIO))


def simulate(qcode="NIKKEI/INDEX", price_type=PriceType.close, way=None, **kw):
    if not isinstance(price_type, enum.Enum):
        price_type = PriceType(price_type)
    df = get_from_quandl(qcode, last_date=None)
    series = getattr(df, price_type.name)
    return RollingMean(series, ratio=kw.get("ratio", DEFAULT_ROLLING_MEAN_RATIO)).simulate()



# TODO: use sqlite3
def get_from_quandl(qcode, last_date=None, price_type=PriceType.close):
    session = models.Session()
    data = session.query(models.Price).filter_by(qcode=qcode).first()
    if data:
        query = session.query(models.Price).filter_by(qcode=qcode)
        return pd.read_sql(query.statement, query.session.bind, index_col="date")
    else:
        mydata = quandl.get(qcode)
        mydata = mydata.rename(columns=MAP_TO_COLUMNS)
        # series = getattr(df, price_type.name)
        mydata = mydata[pd.isnull(mydata.open) == False]
        mydata['qcode'] = qcode
        mydata.to_sql("price", models.engine, if_exists='append')
        return mydata


def show(kw):
    from stock.service.simulate import RollingMean
    qcode = "NIKKEI/INDEX"
    s = models.Session()
    data = s.query(models.Price).filter_by(qcode=qcode).all()
    if data:
        query = s.query(models.Price).filter_by(qcode=qcode)
        mydata = pd.read_sql(query.statement, query.session.bind, index_col="date")
    else:
        mydata = quandl.get(qcode)
        # mydata[mydata.columns[0]]
        columns = {
            "Open Price": "open",
            "Close Price": "close",
            "High Price": "high",
            "Low Price": "low",
        }
        mydata = mydata.rename(columns=columns)
        mydata = mydata[pd.isnull(mydata.open) == False]
        mydata['qcode'] = qcode
        mydata.to_sql("price", models.engine, if_exists='append')  # not need to commit

    series = mydata.open
    print(RollingMean(series, ratio=kw.get("ratio")).simulate())



# coding: utf-8
import os
import logging
from collections import OrderedDict


def set(**kw):
    g = globals()
    for k, v in kw.items():
        g[k.upper()] = v  # raise KeyError if the key does not exist


# Heroku can't import path (I don't know the reason)
# import path
# ROOTDIR = path.Path(__file__).parent.parent
ROOTDIR = "/".join(os.path.abspath(__file__).split("/")[:-2])

HERE = os.path.dirname(__file__)

DEBUG = False
LOG_LEVEL = logging.INFO

HOST = "localhost"
PORT = int(os.environ.get('PORT', 5000))

# sqlalchemy
DATABASE_URL = os.environ.get("DATABASE_URL", "sqlite:///%s/db.sqlite3" % HERE)
CREATE_ENGINE = {
    "encoding": 'utf-8',
    "echo": DEBUG,
}

STATIC_DIR = os.path.join(ROOTDIR, "static")

GRAPH_DIR = os.path.join(STATIC_DIR, "company")

FORMAT = {
    "image_dir": os.path.join(GRAPH_DIR, "{code}"),
    "month": "{code}/{year}_{month}",
}


# frequently changed
COMPANY_XLS_URL = "http://www.jpx.co.jp/markets/statistics-equities/misc/tvdivq0000001vg2-att/data_j.xls"
EXCEL_COMPANY_HEADER = OrderedDict([
    ("date", u'日付'),
    ("code", u'コード'),
    ("name", u'銘柄名'),
    ("_item", u'市場・商品区分'),
    ("category33", u'33業種コード'),
    ("label33", u'33業種区分'),
    ("category17", u'17業種コード'),
    ("label17", u'17業種区分'),
    ("scale", u'規模コード'),
    ("label_scale", u'規模区分'),
])
SHEET_NAME = "Sheet1"

# 今日から株価を取得する期間
DEFAULT_DAYS_PERIOD = 3 * 30



# coding: utf-8
import sqlalchemy as sql

from stock import query


class Session(object):

    def __init__(self, ignore=False, each=False):
        self._s = query.models.Session()

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.close()

    def commit(self):
        self._s.commit()

    def close(self):
        self._s.close()

    def each(self, iterable, add_instance):
        for i in iterable:
            self._s.add(add_instance(**i))
            if not self.each:
                continue
            try:
                self.commit()
            except sql.exc.IntegrityError:
                self._s.rollback()
        if not self.each:
            try:
                self.commit()
            except sql.exc.IntegrityError as e:
                if not self.gnore:
                    raise e

    def with_session(self, f, col_name):
        from stock.service.day_info import make_data_frame
        session = self._s
        for c in query.Company.query(session):
            df = make_data_frame(query.DayInfo.get(company_id=c.id, session=session))
            value = f(df)
            if value is None:
                continue
            if c.search_field is None:
                sf = query.models.CompanySearchField()
            else:
                sf = c.search_field
            setattr(sf, col_name, value)
            c.search_field = sf
            session.add(c)
        session.commit()
        session.close()



# coding: utf-8
import io

import requests
from flask import abort

from stock import query
from stock import models
from stock import config as C

from .session import Session

from logging import getLogger
logger = getLogger(__name__)


def get_scraper():
    from stock.scrape import YahooJapan
    return YahooJapan()


def update_copmany(id, start=None, end=None):
    with Session() as s:
        c = query.Company.query(session=s).first(id=id)
        _update_copmany(s, c, start=start, end=end)


def _update_copmany(session, company, start=None, end=None):
    if not company:
        logger.info("SKIP: company(id=%s)" % company.id)
        return

    def add_instance(**kw):
        kw["company_id"] = company.id
        return models.DayInfo(**kw)

    # 1日だけ更新する場合でも複数ページにアクセスしている(無駄)
    history = get_scraper().history(company.code, start, end)
    session.each(history, add_instance)


def update_copmany_list(start=None, end=None, each=False, ignore=False, last_date=None, limit=None):
    session = Session(ignore=ignore, each=each)
    for c in query.Company.all(last_date=last_date, session=session._s, limit=limit):
        _update_copmany(session, c)
    session.close()


def download_company_list(url=C.COMPANY_XLS_URL):
    from stock.cmd.import_company import Reader
    resp = requests.get(url)
    if resp.ok:
        return Reader(filepath=io.BytesIO(resp.content))


def download_and_store_company_list(url=C.COMPANY_XLS_URL):
    reader = download_company_list(url)
    if reader:
        return reader.store()
    else:
        return False


def first(id, raise_404=False):
    company = query.Company.first(id=id)
    if not company and raise_404:
        abort(404)
    return company


def _percent(q, kw, col_name):
    if col_name not in kw or kw[col_name] is None:
        return q
    val = kw[col_name]
    col = getattr(query.models.CompanySearchField, col_name)
    if val >= 0:
        q = q.filter(col >= val)
    else:
        q = q.filter(col < val)
    return q


def _cross(q, kw, val_name, format_col_name):
    if val_name not in kw or kw[val_name] is None:
        return q

    col_name1 = format_col_name % 1
    col_name2 = format_col_name % 2
    v = kw[val_name]
    col1 = getattr(query.models.CompanySearchField, col_name1)
    col2 = getattr(query.models.CompanySearchField, col_name2)
    if v > 0:
        # Golden cross
        q = q.filter(col1 >= 0)
        q = q.filter(col2 <= 0)
    else:
        # Dead cross
        q = q.filter(col1 <= 0)
        q = q.filter(col2 >= 0)
    return q


def get(**kw):
    session = query.models.Session()
    q = query.Company.query(session)
    if any(kw.values()):
        q = q.join(query.models.Company.search_field)

    q = _percent(q, kw, "ratio_closing_minus_rolling_mean_25")
    q = _percent(q, kw, "ratio_closing1_minus_closing2")

    q = _cross(q, kw, "closing_stochastic_d_minus_sd", "closing_stochastic_d_minus_sd%d_14_3_3")
    q = _cross(q, kw, "closing_macd_minus_signal", "closing_macd_minus_signal%d_26_12_9")

    if kw["closing_rsi_14"] is not None:
        rsi = kw["closing_rsi_14"]
        col = query.models.CompanySearchField.closing_rsi_14
        if rsi >= 0:
            q = q.filter(col >= rsi)
        else:
            rsi *= -1
            q = q.filter(col < rsi)

    if kw["interval_closing_bollinger_band_20"] is not None:
        col_name = "interval_closing_bollinger_band_20"
        v = kw["interval_closing_bollinger_band_20"]
        col = getattr(query.models.CompanySearchField, col_name)
        q = q.filter(col == v)

    return q.all()



# coding: utf-8
import datetime

import pandas as pd


def to_seq(dates, series):
    return list([a, b] for a, b
                in zip(dates.values.tolist(), series.values.tolist())
                if not pd.isnull(b))


class DayInfoQuery(object):

    def __init__(self, q):
        self.query = q

    def df(self):
        from stock.service.day_info import make_data_frame
        return make_data_frame(self.query)

    def rolling_mean(self, period=30):
        df = self.df()
        return to_seq(df.date, pd.rolling_mean(df.closing, period))

    def bollinger_band(self, period=20, sigma=2):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.bollinger_band(df.closing, period, sigma))

    def macd_line(self):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.macd_line(df.closing))

    def macd_signal(self):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.macd_signal(df.closing))

    def stochastic_k(self):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.stochastic_k(df.closing, k=14))

    def stochastic_d(self):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.stochastic_d(df.closing, k=14, d=3))

    def stochastic_sd(self):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.stochastic_sd(df.closing, k=14, d=3, sd=3))

    def RSI(self, period):
        from stock.service import chart
        df = self.df()
        return to_seq(df.date, chart.rsi(df.closing, period))

    def ohlc(self):
        return [(d["date"], d["opening"], d["high"], d["low"], d["closing"])
                for d in self.df().to_dict(orient="records")]

    def volume(self):
        df = self.df()
        return to_seq(df.date, df.volume)

    def to_series(self):
        bbands = [{"name": "%dsigma" % s,
                   "data": self.bollinger_band(sigma=s),
                   "color": "black",
                   "yAxis": 0,
                   "lineWidth": 0.5}
                  for s in [3, 2, 1, -1, -2, -3]]

        # tooltip={valueDecimals: 2}
        return bbands + [
            {"name": "rolling_mean25", "data": self.rolling_mean(period=25)},
            {"name": "rolling_mean5", "data": self.rolling_mean(period=5)},
            {"name": "OHLC", "data": self.ohlc(), "type": "candlestick", "yAxis": 0, "color": 'blue'},
            {"name": "volume", "data": self.volume(), "type": "column", "yAxis": 1},
            {"name": "RSI", "data": self.RSI(period=14), "yAxis": 2},
            {"name": "macd_line", "data": self.macd_line(), "yAxis": 3},
            {"name": "macd_signal", "data": self.macd_signal(), "yAxis": 3},
            {"name": "%K", "data": self.stochastic_k(), "yAxis": 4},
            {"name": "%D", "data": self.stochastic_d(), "yAxis": 4},
            {"name": "%SD", "data": self.stochastic_sd(), "yAxis": 4},
        ]


class Wrapper(object):

    def __init__(self, ins):
        from . import models  # Not cyclic import
        assert isinstance(ins, getattr(models, self.__class__.__name__))
        self.ins = ins

    # company_idみたいに、定義されていない属性はinsから取得
    def __getattr__(self, name):
        return getattr(self.ins, name)

    def to_dict(self):
        assert getattr(self, "dict_keys", False), "Must need a dict_keys attribute"
        return {k: getattr(self, k) for k in self.dict_keys}


class Company(Wrapper):
    dict_keys = ["id", "name", "code"]

    def fix_data_frame(self):
        data_records = []
        for day_info in self.day_info_list:
            data = {
                "high": day_info.fix_high(),
                "low": day_info.fix_low(),
                "opening": day_info.fix_opening(),
                "closing": day_info.fix_closing(),
                "date": day_info.js_datetime}
            data_records.append(data)
        return pd.DataFrame.from_records(data_records)


def golden_cross(ins, format_string):
    v1 = getattr(ins, format_string % 1)
    v2 = getattr(ins, format_string % 2)
    return v1 > 0 and v2 < 0


def dead_cross(ins, format_string):
    v1 = getattr(ins, format_string % 1)
    v2 = getattr(ins, format_string % 2)
    return v1 < 0 and v2 > 0


buy_conditions = {
    "closing_rsi_14": lambda ins: ins.closing_rsi_14 <= 30,
    "closing_macd_minus_signal": lambda ins: golden_cross(ins, "closing_macd_minus_signal%s_26_12_9"),
    "ratio_closing_minus_rolling_mean_25": lambda ins: ins.ratio_closing_minus_rolling_mean_25 <= -10,
    "interval_closing_bollinger_band_20": lambda ins: ins.interval_closing_bollinger_band_20 <= -3,
}

sell_conditions = {
    "closing_rsi_14": lambda ins: ins.closing_rsi_14 >= 70,
    "closing_macd_minus_signal": lambda ins: dead_cross(ins, "closing_macd_minus_signal%s_26_12_9"),
    "ratio_closing_minus_rolling_mean_25": lambda ins: ins.ratio_closing_minus_rolling_mean_25 >= 10,
    "interval_closing_bollinger_band_20": lambda ins: ins.interval_closing_bollinger_band_20 >= 3,
}


class CompanySearchField(Wrapper):

    @property
    def number_of_signals(self):
        return len(self.signals)

    def signal(self, col_name):
        buy = buy_conditions[col_name]
        sell = sell_conditions[col_name]
        try:
            if buy(self.ins):
                return "BUY"
            elif sell(self.ins):
                return "SELL"
        except TypeError:
            pass
        return ""

    def signals(self):
        # TODO: implement
        s = {}
        for meth in dir(self):
            prefix = "signal_"
            if meth.startswith(prefix):
                s[meth[len(prefix):]] = getattr(self, meth)()
        return s


class DayInfo(Wrapper):

    dict_keys = ["high", "low", "opening", "closing", "company_id", "js_datetime"]

    def _fix_value(self, value):
        for date in self.ins.company.split_stock_dates:
            if self.date < date.date:
                value *= date.from_number / float(date.to_number)
        return value

    @property
    def high(self):
        return self._fix_value(self.ins.high)

    @property
    def low(self):
        return self._fix_value(self.ins.low)

    @property
    def opening(self):
        return self._fix_value(self.ins.opening)

    @property
    def closing(self):
        return self._fix_value(self.ins.closing)

    @property
    def js_datetime(self):
        japan = self.ins.date + datetime.timedelta(hours=9)
        return int(japan.strftime("%s")) * 1000



# coding: utf-8
import pandas as pd

from .session import Session
from stock.service import chart


def last(series, offset_from_last=0):
    i = series.last_valid_index()
    if i is None:
        return
    elif i - offset_from_last >= 0:
        return series[i - offset_from_last]


def increment(a, b):
    if a is None or b is None:
        return
    elif a.is_integer() and b.is_integer():
        return float((a - b) / b) * 100


def with_session(f, col_name):
    with Session() as s:
        s.with_session(f, col_name)


def update_search_fields():
    closing_minus_rolling_mean_25()
    closing_rsi_14()
    closing_macd_minus_signal()
    low_min()
    closing_stochastic_d_minus_sd()
    ratio_sigma_low_minus_closing()


def closing_minus_rolling_mean_25(period=25):
    """長期移動平均線と現在の株価の差を予め計算"""
    def f(df):
        mean = pd.rolling_mean(df.closing, period)
        return increment(last(df.closing), last(mean))

    with_session(f, "ratio_closing_minus_rolling_mean_25")


def closing_rsi_14(period=14):
    """営業最終日のRSIを求める"""
    def f(df):
        rsi = chart.rsi(df.closing, period)
        if not rsi.empty:
            return float(rsi[rsi.last_valid_index()])
    with_session(f, "closing_rsi_14")


def closing_macd_minus_signal():
    def wrap(index):
        def last(p):
            lvi = p.last_valid_index()
            if lvi and lvi > 0:
                return p[lvi - (index - 1)]

        def f(df):
            macd = last(chart.macd_line(df.closing))
            signal = last(chart.macd_signal(df.closing))
            if macd is not None and signal is not None:
                return float(macd - signal)
        return f
    with_session(wrap(1), "closing_macd_minus_signal1_26_12_9")
    with_session(wrap(2), "closing_macd_minus_signal2_26_12_9")


def low_min():
    def wrap(index):
        def f(df):
            v = float(df.low.tail(index).min())
            if not pd.isnull(v):
                return v
        return f
    with_session(wrap(25), "low_min_25")
    with_session(wrap(75), "low_min_75")
    with_session(wrap(200), "low_min_200")


def closing_bollinger_band(period=20):
    def f(df):
        prices = df.closing
        if prices.empty:
            return None
        p = float(prices.tail(1))
        mean = float(pd.rolling_mean(prices, period).tail(1))
        std = float(pd.rolling_std(prices, period).tail(1))
        if pd.isnull(mean) or pd.isnull(std):
            return None
        if p == mean:
            return 0
        sign = 1 if p > mean else -1
        for sigma in [1, 2, 3]:
            m1 = mean + sign * std * (sigma - 1)
            m2 = mean + sign * std * sigma
            if min(m1, m2) < p <= max(m1, m2):
                return sign * sigma
    with_session(f, "interval_closing_bollinger_band_20")


def closing_stochastic_d_minus_sd():
    def wrap(index):
        def last(p):
            lvi = p.last_valid_index()
            if lvi and lvi > 0:
                return p[lvi - (index - 1)]

        def f(df):
            k, d, sd = 14, 3, 3
            fast = last(chart.stochastic_d(df.closing, k=k, d=d))
            slow = last(chart.stochastic_sd(df.closing, k=k, d=d, sd=sd))
            if fast is not None and slow is not None:
                return float(fast - slow)
        return f
    with_session(wrap(1), "closing_stochastic_d_minus_sd1_14_3_3")
    with_session(wrap(2), "closing_stochastic_d_minus_sd2_14_3_3")


def ratio_closing1_minus_closing2():
    def f(df):
        c0 = last(df.closing)
        c1 = last(df.closing, 1)
        return increment(c0, c1)

    with_session(f, "ratio_closing1_minus_closing2")


def ratio_sigma_low_minus_closing(period=20):
    """
    安値と前の日の終値の割合を求める。
    ボリンジャーバンドの考え方を応用して、次の日の安値を求める.
    """
    def wrap(index):
        def f(df):
            c = df.closing.shift(1)
            l = df.low
            d = (l - c)
            d = d[d < 0][:period]
            f = float(d.mean() - index * d.std())
            li = c.last_valid_index()
            if li:
                l = float(c[li])
                if not pd.isnull(f) and not pd.isnull(l):
                    return l + f
        return f
    with_session(wrap(1), "ratio_1sigma_low_minus_closing")
    with_session(wrap(2), "ratio_2sigma_low_minus_closing")



import enum
from collections import OrderedDict
import pandas as pd
from stock.service import chart


DEFAULT_FIGSIZE = (10, 10)


class LineType(enum.Enum):

    rooling_mean = "rooling_mean"


class Action(enum.IntEnum):

    BUY = 1
    SELL = 2
    LOSTCUT = 3
    FORCE = 4
    NANPIN = 5

    @property
    def color(self):
        return dict(zip(Action, "rbgym")).get(self.value, "r")

    @classmethod
    def xlabel(cls):
        return 'color : %s' % ", ".join("{0.name} = {0.color}".format(a) for a in cls)

Timing = enum.IntEnum("Timing", "BUY SELL")


def increment_ratio(a2, a1):
    return ((a2 - a1) / a1) * 100


def timing(*series):
    return pd.concat(series, axis=1).T.apply(mapping)


def mapping(x):
    y = x[x.notnull()]
    y0 = y.ix[0]
    # if (y0 == y).all():  # error
    if (y == y0).all():
        return y0
    return None


class Status(object):

    def __init__(self, simulator):
        self.simulator = simulator
        self.current = None
        self.accumulation = 0
        self.action = None

    def to_dict(self):
        return {
            "current": self.current,
            "accumulation": self.accumulation,
            "action": self.action,
        }

    def is_lostcut(self, date):
        if self.current is None:
            return False
        price = self.simulator.series[date]
        return increment_ratio(price, self.current) <= -self.simulator.lostcut

    def finish(self):
        last = self.simulator.series.last_valid_index()
        self.sell(last)
        self.action = Action.FORCE

    def buy(self, date):
        self.current = self.simulator.series[date]
        self.accumulation -= self.simulator.series[date]
        self.action = Action.BUY

    def sell(self, date):
        # TODO: 売買手数料を考慮
        self.current = None
        self.accumulation += self.simulator.series[date]
        self.action = Action.SELL

    def lostcut(self, date):
        self.sell(date)
        self.action = Action.LOSTCUT

    def is_buy(self, date):
        if date not in self.simulator.timing:
            return False
        last = self.simulator.series.last_valid_index()
        if date == last:
            return False
        return (self.action in [None, Action.SELL, Action.LOSTCUT]) and self.simulator.timing[date] == Timing.BUY

    def is_sell(self, date):
        if self.current is None:
            return False
        if date not in self.simulator.timing:
            return False
        return self.action in [Timing.BUY] and self.simulator.timing[date] == Timing.SELL

    def is_finish(self):
        return self.current is not None


class Simulator(object):

    # ロストカットした場合は、トレンドの変化を確認してリセットする必要あり
    # 指標によっては、損切りした直後に買いシグナルが出る場合があるから
    def __init__(self,
                 series,
                 timing,
                 lostcut=3,
    ):
        self.series = series
        self.timing = timing
        self.lostcut = lostcut

    def set_ax(self, ax):
        ax.set_title('Simulator')  # TODO: show qcode
        df = self.simulate()
        ymin, ymax = ax.get_ylim()
        for action in Action:
            signal = df.action[df.action == action]
            ax.vlines(x=signal.index, ymin=ymin, ymax=ymax - 1, color=action.color)
        ax.set_xlabel(Action.xlabel())
        return ax

    def simulate(self):
        s = self._simulate()
        return s
        return pd.concat([s, self.series], axis=1)

    def _simulate(self):
        result = []
        dates = []
        status = Status(simulator=self)
        for date in OrderedDict(self.series):
            if status.is_lostcut(date):
                status.lostcut(date)
            elif status.is_buy(date):
                status.buy(date)
            elif status.is_sell(date):
                status.sell(date)
            else:
                continue
            result.append(status.to_dict())
            dates.append(date)
        if status.is_finish():
            status.finish()
            result.append(status.to_dict())
            dates.append(date)
        return pd.DataFrame(result, index=dates)


class SignalLine(object):

    def __repr__(self):
        return self.__str__()

    def plot(self, figsize=DEFAULT_FIGSIZE, **kw):
        return self.df.plot(figsize=figsize)

    def set_ax(self, ax):
        ax.set_title('Rolling Mean')
        ymin, ymax = ax.get_ylim()
        ax.vlines(x=self.buy.index, ymin=ymin, ymax=ymax-1, color='r')
        ax.vlines(x=self.sell.index, ymin=ymin, ymax=ymax-1, color='b')
        return ax

    @property
    def timing(self):
        return timing(self.buy, self.sell)

    def simulate(self):
        return Simulator(self.series, self.timing).simulate()

    def simulate_action(self):
        df = Simulator(self.series, self.timing).simulate()
        if df.empty:
            return df
        notnull = df[df.action.notnull()]
        s = notnull['action'].map(lambda x: None if pd.isnull(x) else Action(x).name)
        return pd.concat([df.ix[s.index], s], axis=1)

    def simulate_plot(self):
        return Simulator(self.series, self.timing).set_ax(self.plot())


# ratio = [0, 100] => (1, 25)  # 最大幅以下しか取れない
# period = 25 or 30 days?  # 短期から長期にかけて計算できるようにする

class RollingMean(SignalLine):

    def __init__(self, series, ratio=10, period=25):
        self.series = series
        self.period = period
        self.ratio = ratio

    def __str__(self):
        return "RollingMean(period={period}, ratio={ratio})".format(**self.__dict__)

    __repr__ = __str__

    @property
    def df(self):
        return pd.concat({
            "series": self.series,
            "mean": self.mean
        }, axis=1)

    @property
    def mean(self):
        return self.series.rolling(center=False, window=self.period).mean()

    @property
    def iratio(self):
        return increment_ratio(self.mean, self.series)

    @property
    def buy(self):
        x = self.iratio
        return x[x >= self.ratio].map(lambda x: Timing.BUY)

    @property
    def sell(self):
        x = self.iratio
        return x[-self.ratio >= x].map(lambda x: Timing.SELL)


class MACD(SignalLine):

    def __init__(self, series, fast=26, slow=12, signal=9):
        self.series = series
        self.fast = fast
        self.slow = slow
        self.signal = signal

    def __str__(self):
        return "MACD(fast={fast}, slow={slow}, signal={signal})".format(**self.__dict__)

    @property
    def df(self):
        # 単位が違うので、別々に表示する必要あり
        return pd.concat({
            "series": self.series,
            "macd": self.macd,
            "signal": self.signal_line,
        }, axis=1)

    @property
    def macd(self):
        return chart.macd_line(**self.__dict__)

    @property
    def signal_line(self):
        return chart.macd_signal(**self.__dict__)

    @property
    def cross(self):
        d = self.macd - self.signal_line
        return d[d * d.shift(1) < 0]

    @property
    def buy(self):
        c = self.cross
        return c[c > 0].map(lambda x: Timing.BUY)

    @property
    def sell(self):
        c = self.cross
        return c[c < 0].map(lambda x: Timing.SELL)



# coding: utf-8
from logging import getLogger

import requests
import bs4

from stock.util import DateRange

logger = getLogger(__name__)


class ParseError(Exception):
    pass


def clean_value(value):
    """文字列の数字を整数に変換する"""
    value = value.replace(",", "")
    return float(value)


class Scraper(object):

    def request(self, **kw):
        url = self.get_url(**kw)
        logger.info("GET: %s" % url)
        try:
            res = requests.get(url)
        except OSError:
            return None
        soup = self._get_soup(res)
        try:
            values = self.parse(soup)
        except ParseError as e:
            logger.warning("CAN NOT PARSE: %s" % e)
            return None
        return self.clean(values)

    def get_url(self, **kw):
        return self.url.format(**kw)

    def _get_soup(self, response):
        html = response.content.decode("utf-8").encode("utf-8")
        return bs4.BeautifulSoup(html, "html.parser")

    def parse(self, soup):
        raise NotImplementedError

    def clean(self, values):
        return values


class Meta(type):

    def __new__(cls, name, bases, attrs):
        for s in attrs.get("scrapers", []):
            attrs[s.__name__] = s
        return super(Meta, cls).__new__(cls, name, bases, attrs)


class Interface(object, metaclass=Meta):

    def current_value(self, code):
        """ :return: int """
        return self.clean_current_value(self.CurrentValue().request(code=code))

    def day_info(self, code):
        """
        :return:
            {"date": 株価の日付
             "opening": 始値,
             "high": 高値,
             "low": 安値,
             "closing": 終値,
             "volume": 出来高}
        """
        return self.clean_day_info(self.DayInfo().request(code=code))

    def _each_page(self, code, start, end, page, scraper):
        each = []
        for p in range(1, page + 1):
            r = scraper.request(
                code=code, p=p, **DateRange(start, end).to_short_dict())
            if r:
                each += r
            else:
                break
        return self.clean_history(each)

    def split_stock_date(self, code, start=None, end=None, page=100):
        """
        :return:
            {"from_number": 株式分割前,
             "to_number": 株式分割後,
             "date": 分割日}
        """
        return self._each_page(code, start, end, page, self.SplitStockDate())

    def history(self, code, start=None, end=None, page=100):
        """:return: day_infoの戻り値と同じKEY-VALUEをもつ辞書のリスト"""
        return self._each_page(code, start, end, page, self.History())

    def clean_current_value(self, current_value):
        return clean_value(current_value)

    def clean_day_info(self, day_info):
        if day_info:
            return {k: clean_value(v) if k != "date" else v for k, v in day_info.items()}

    def clean_history(self, history):
        return [self.clean_day_info(d) for d in history if d]



# coding: utf-8
import time
import datetime

import sqlalchemy as sql
from dateutil import relativedelta

from . import config as C


class DateRange(object):

    def __init__(self, start=None, end=None):
        if isinstance(end, str):
            end = str2date(start)
        if isinstance(start, str):
            start = str2date(start)
        if end is None:
            end = datetime.date.today()
        if start is None:
            start = end - relativedelta.relativedelta(days=C.DEFAULT_DAYS_PERIOD)
        self.end = end
        self.start = start

    def query(self, date_col):
        start, end = self.start, self.end
        if start is None:
            return date_col <= end
        elif end is None:
            return start <= date_col
        else:
            return sql.and_(start <= date_col, date_col <= end)

    def to_dict(self):
        return {"start": str(self.start), "end": str(self.end)}

    def to_short_dict(self):
        return {
            "sy": self.start.year,
            "sm": self.start.month,
            "sd": self.start.day,
            "ey": self.end.year,
            "em": self.end.month,
            "ed": self.end.day,
        }


def dict_inverse(dct):
    return {v: k for k, v in dct.items()}


def str2date(datestr):
    if datestr:
        t = time.strptime(datestr, "%Y-%m-%d")
        return datetime.date.fromtimestamp(time.mktime(t))



# coding: utf-8
from logging import getLogger

import sqlalchemy as sql

from . import models
from . import util

logger = getLogger(__name__)


class Query(object):
    model = None

    @classmethod
    def session(cls):
        return models.Session()

    @classmethod
    def query(cls, session=None):
        if session is None:
            session = cls.session()
        return session.query(cls.model)

    @classmethod
    def one(cls, id, session=None):
        return cls.query(session).filter_by(id=id).one()

    @classmethod
    def first(cls, session=None, **kw):
        return cls.query(session).filter_by(**kw).first()


class DayInfo(Query):
    model = models.DayInfo

    @classmethod
    def get(cls, company_id, start=None, end=None, session=None):
        q = cls.query(session).filter_by(company_id=company_id)
        q = q.filter(util.DateRange(start, end).query(cls.model.date))
        q = q.order_by("date")
        return q


class Company(Query):
    model = models.Company

    @classmethod
    def all(cls, session=None, last_date=None, limit=None, **kw):
        q = cls.query(session).filter_by(**kw)
        if limit:
            q = q.limit(limit)
        if last_date:
            q = q.filter(sql.not_(models.Company.day_info_list.any(date=last_date)))
        return q.all()

    @classmethod
    def first(cls, session=None, last_date=None, **kw):
        q = cls.query(session).filter_by(**kw)
        if last_date:
            q = q.filter(sql.not_(models.Company.day_info_list.any(date=last_date)))
        return q.first()

    @classmethod
    def max_id(cls):
        q = cls.query()
        q = q.order_by("-id")
        c = q.first()
        return c.id if c else 0



# coding: utf-8
from setuptools import setup, find_packages
from pip.req import parse_requirements

install_requires = [str(ir.req) for ir in parse_requirements("requirements.txt", session=False)]

setup(
    name='stock',
    version='0.6',
    author='Hiroyuki Ishii',
    author_email="hiroyuki.ishii.42@gmail.com",
    url="http://f1nance.herokuapp.com/",
    py_modules=['stock'],
    packages=find_packages(),
    description='A command line tool for stock markets',
    dependency_links=[],
    keywords=['stock'],
    install_requires=install_requires,
    entry_points='''
        [console_scripts]
        stock=stock.cmd:cli
    ''',
)



# coding: utf-8
import click

from stock import scrape
from stock import util
from stock import service
from stock import config as C


def _multiple_decorator(funcs):
    def wrap(g):
        for f in funcs:
            g = f(g)
    return wrap


def mkdate(ctx, param, datestr):
    return util.str2date(datestr)


class AliasedGroup(click.Group):

    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))


@click.group(cls=AliasedGroup, invoke_without_command=True)
def cli():
    ctx = click.get_current_context()
    if ctx.invoked_subcommand is None:
        ctx.invoke(serve)


@cli.group()
def store():
    pass


@click.option("--url", default=C.COMPANY_XLS_URL)
@store.command(help="""\
You can download the xls at \
http://www.jpx.co.jp/markets/statistics-equities/misc/01.html
or directly {url}
""".format(url=C.COMPANY_XLS_URL))
def company(url):
    if service.company.download_and_store_company_list(url):
        click.echo("Get %s" % url)
    else:
        click.echo("Can't get %s" % url)


@cli.group()
def db():
    pass


@db.command(help="Create new all tables")
def create():
    service.table.create()


@db.command(help="Drop all tables")
@click.option("-y", "yes", is_flag=True, default=False)
def drop(yes):
    if yes or click.confirm("Drop all tables. Are you sure?"):
        service.table.drop()


@cli.command(help="Setup")
@click.option("-l", "--limit", type=int, default=10)
@click.option("--all", is_flag=True, default=False)
def setup(all, limit):
    if all:
        limit = None
    click.echo("Start setup ...")
    click.echo("Create table ...")
    service.table.create()
    click.echo("Download and store company list ...")
    service.company.download_and_store_company_list()
    click.echo("Store day info")
    service.company.update_copmany_list(limit=limit, each=True, ignore=True)
    click.echo("Update search fields")
    service.search_field.update_search_fields()


@cli.command(help="show companies")
@click.option("-c", "--closing-minus-rolling-mean-25", type=int)
def show(closing_minus_rolling_mean_25):
    iters = service.get_companies(
        closing_minus_rolling_mean_25=closing_minus_rolling_mean_25
    )
    for company in iters:
        click.echo(company)
    click.echo("Summary")
    click.echo("count: %s" % len(iters))


@click.option("--port", default=C.PORT, type=int)
@click.option("--host", default=C.HOST)
@click.option("-l", "--log-level", default=C.LOG_LEVEL, type=int)
@click.option("--debug", default=(not C.DEBUG), is_flag=True)
@cli.command(help="Start server")
def serve(**kw):
    from stock.server import app
    import logging
    msg = ", ".join(["%s = %s" % (k, v) for k, v in kw.items()])
    click.echo(msg)
    C.set(**kw)
    click.echo("DATABASE_URL: %s" % C.DATABASE_URL)
    logging.basicConfig(level=kw.pop("log_level"))
    app.run(**kw)


@cli.group(name="scrape")
def scrape_():
    pass

_decorator = _multiple_decorator([
    click.option("--scraper", default=scrape.YahooJapan),
    click.option("--start"),
    click.option("--end"),
    click.argument('code', type=int)
])


@_decorator
@scrape_.command(name="history")
def scrape_history(code, start, end, scraper):
    history = scraper.history(code, start, end)
    for day_info in history:
        click.echo(day_info)


@_decorator
@scrape_.command(name="split")
def split_stock_date(code, start, end, scraper):
    history = scraper.split_stock_date(code, start, end)
    for day_info in history:
        click.echo(day_info)


@click.option("--scraper", default=scrape.YahooJapan)
@click.argument('code', type=int)
@scrape_.command(name="day_info")
def day_info(code, scraper):
    day_info = scraper.day_info(code)
    click.echo(day_info)


@click.option("--scraper", default=scrape.YahooJapan)
@click.argument('code', type=int)
@scrape_.command(name="value")
def current_value(code, scraper):
    value = scraper.current_value(code)
    click.echo(value)


@click.option("--limit", type=int, default=None)
@click.option("--each", is_flag=True, default=False)
@click.option("--no-field", is_flag=True, default=True)
@click.option("--last-date", callback=mkdate)
@cli.command(help="update day info")
def update(each, last_date, limit, no_field):
    if last_date is None:
        last_date = service.util.last_date()
    click.echo("update company list")
    service.company.update_copmany_list(each=True, ignore=True, last_date=last_date)
    if no_field:
        click.echo("update search fields")
        service.search_field.update_search_fields()


@cli.command(help="calculate everything", name="calc")
@click.option("--all", is_flag=True, default=False)
@click.option("--rcmc", "ratio_closing1_minus_closing2", is_flag=True, default=False)
@click.option("--bband", "closing_bollinger_band", is_flag=True, default=False)
@click.option("--rm25", "closing_minus_rolling_mean_25", is_flag=True, default=False)
@click.option("--rsi", "closing_rsi_14", is_flag=True, default=False)
@click.option("--min", "low_min", is_flag=True, default=False)
@click.option("-D", "closing_stochastic_d_minus_sd", is_flag=True, default=False)
@click.option("--macd", "closing_macd_minus_signal", is_flag=True, default=False)
@click.option("--lmc", "ratio_sigma_low_minus_closing", is_flag=True, default=False)
def calculate(**kw):
    # need to pop if it is not a service
    all = kw.pop("all")
    for method_name, do in kw.items():
        if all or do:
            meth = getattr(service.search_field, method_name, None)
            if meth:
                click.echo("CALC: %s" % method_name)
                meth()
            else:
                click.secho("CALC: %s is not found" % method_name, fg='red')


@cli.command(help="simulate", name="sm")
@click.argument('q', '--quandl-code', default="NIKKEI/INDEX")
@click.option("-p", '--price-type', default="close")
@click.option("--ratio", type=float)
@click.option("--lostcut", type=float)
@click.option("--start")
@click.option("--end")
def simulate(**kw):
    from stock.service.table import s
    s(**kw)


if __name__ == "__main__":
    cli()
